@@ -14,12 +14,12 @@ our @ISA = qw(Exporter);
 # This allows declaration	use Device::NeurioTools ':all';
 # If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
 # will save memory.
+
 our %EXPORT_TAGS = ( 'all' => [ qw(
-    new connect set_rate get_cost get_kwh
+    new get_cost get_energy_consumed get_kwh_consumed get kwh_generated get_power_consumed get_rate get_timezone set_rate set_timezone
 ) ] );
 
 our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
-
 our @EXPORT = qw();
 
 BEGIN
@@ -40,15 +40,16 @@ BEGIN
 
 =head1 NAME
 
-Device::NeurioTools - More complex methods for accessing data collected by a Neurio sensor module.
+Device::NeurioTools - More complex methods for accessing data collected by a 
+                      Neurio sensor module.
 
 =head1 VERSION
 
-Version 0.04
+Version 0.05
 
 =cut
 
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 
 #*****************************************************************
 
@@ -66,7 +67,7 @@ our $VERSION = '0.04';
  (key, secret, sensor_id) as well as an Energy Aware Neurio sensor installed in
  your house.
 
- The module is written entirely in Perl and has been tested on Raspbian Linux.
+ The module is written entirely in Perl and has been developped on Raspbian Linux.
 
 
 =head1 SAMPLE CODE
@@ -74,18 +75,21 @@ our $VERSION = '0.04';
     use Device::Neurio;
     use Device::NeurioTools;
 
-    my $my_Neurio = Device::Neurio->new($key,$secret,$sensor_id);
+    $my_Neurio = Device::Neurio->new($key,$secret,$sensor_id);
 
     $my_Neurio->connect();
 
-    my $my_NeurioTools = Device::NeurioTools->new($my_Neurio);
+    $my_NeurioTools = Device::NeurioTools->new($my_Neurio);
 
     $my_NeurioTools->set_timezone();
     $my_NeurioTools->set_rate(0.08);
-    my $kwh = $my_NeurioTools->get_kwh("2014-06-24T00:00:00".$my_NeurioTools->get_timezone(),
-                                       "minutes",
-                                       "2014-06-24T23:59:59".$my_NeurioTools->get_timezone(),
-                                       "5");
+    
+    $start = "2014-06-24T00:00:00".$my_NeurioTools->get_timezone();
+    $end   = "2014-06-24T23:59:59".$my_NeurioTools->get_timezone();
+    $kwh   = $my_NeurioTools->get_kwh($start,"minutes",$end,"5");
+
+    undef $my_NeurioTools;
+    undef $my_Neurio;
 
 =head2 EXPORT
 
@@ -194,10 +198,11 @@ sub set_timezone {
     if (defined $offset) {
 	  $total = $offset;
     } else {
-      my @utc   = gmtime();
-      my @loc   = localtime();
-      $total = ($loc[2]*60+$loc[1])-($utc[2]*60+$utc[1]);
+      my @utc = gmtime();
+      my @loc = localtime();
+      $total  = ($loc[2]*60+$loc[1])-($utc[2]*60+$utc[1]);
     }
+    
     $hours = sprintf("%+03d",$total / 60);
     $mins  = sprintf("%02d",$total % 60);
     $self->{'timezone'} = "$hours:$mins";
@@ -228,11 +233,13 @@ sub get_timezone {
 
 #*****************************************************************
 
-=head2 get_cost - calculate the cost of consumption for the specified period
+=head2 get_cost - calculate the cost of consumed power for the specified period
 
- Calculates the cost of consumption over the period specified.
+ Calculates the cost of consumed power over the period specified.
 
    $NeurioTools->get_cost($start,$granularity,$end,$frequency);
+   
+   This method requires that a 'rate' be set using the set_rate() method
  
    This method accepts the following parameters:
      - start       : yyyy-mm-ddThh:mm:ssZ - Required
@@ -247,12 +254,12 @@ sub get_cost {
     my ($self,$start,$granularity,$end,$frequency) = @_;
     my $i=0;
     
-    if ($self->{'rate'} ==0 ) {
+    if ($self->{'rate'} == 0 ) {
         print "NeurioTools->get_cost(): Cannot calculate cost since rate is set to zero\n";
         return 0;
     }
     
-    my $kwh  = $self->get_kwh($start,$granularity,$end,$frequency);
+    my $kwh  = $self->get_kwh_consumed($start,$granularity,$end,$frequency);
     my $cost = $kwh*$self->{'rate'};
     
     return $cost;
@@ -261,11 +268,11 @@ sub get_cost {
 
 #*****************************************************************
 
-=head2 get_kwh - calculate the kwh of consumption for the specified period
+=head2 get_kwh_consumed - kwh of consumed power for the specified period
 
- Calculates the kwh of consumption over the period specified.
+ Calculates the total kwh of consumed power over the period specified.
 
-   $NeurioTools->get_kwh($start,$granularity,$end,$frequency);
+   $NeurioTools->get_kwh_consumed($start,$granularity,$end,$frequency);
  
    This method accepts the following parameters:
      - start       : yyyy-mm-ddThh:mm:ssZ - Required
@@ -278,20 +285,62 @@ sub get_cost {
  Returns the kwh on success 
  Returns 0 on failure
 =cut
-sub get_kwh {
+sub get_kwh_consumed {
+    my ($self,$start,$granularity,$end,$frequency) = @_;
+    my $energy  = 0;
+    my $samples = 0;
+    my $kwh;
+    
+    my $data      = $self->{'neurio'}->fetch_Energy_Stats($start,$granularity,$end,$frequency,"1","5000");
+    my $start_obj = DateTime::Format::ISO8601->parse_datetime($start);
+    my $end_obj   = DateTime::Format::ISO8601->parse_datetime($end);
+    my $dur_obj   = $end_obj->subtract_datetime($start_obj);
+    my $duration  = eval($dur_obj->{'minutes'}*60+$dur_obj->{'seconds'});
+    
+    while (defined $data->[$samples]->{'consumptionEnergy'}) {
+        $energy = $energy + $data->[$samples]->{'consumptionEnergy'};
+        $samples++;
+    }
+    
+    $kwh = $energy/(1000*3600);
+
+    return $kwh;
+}
+
+
+#*****************************************************************
+
+=head2 get_kwh_generated - kwh of generated power for the specified period
+
+ Calculates the total kwh of generated power over the period specified.
+
+   $NeurioTools->get_kwh_generated($start,$granularity,$end,$frequency);
+ 
+   This method accepts the following parameters:
+     - start       : yyyy-mm-ddThh:mm:ssZ - Required
+                     specified using ISO8601 format
+     - granularity : seconds|minutes|hours|days - Required
+     - end         : yyyy-mm-ddThh:mm:ssZ - Optional
+                     specified using ISO8601 format
+     - frequency   : an integer - Optional
+ 
+ Returns the kwh on success 
+ Returns 0 on failure
+=cut
+sub get_kwh_generated {
     my ($self,$start,$granularity,$end,$frequency) = @_;
     my $samples = 0;
     my $power   = 0;
     my $kwh;
     
-    my $data = $self->{'neurio'}->fetch_Samples($start,$granularity,$end,$frequency);
-    
+    my $data      = $self->{'neurio'}->fetch_Samples($start,$granularity,$end,$frequency);
     my $start_obj = DateTime::Format::ISO8601->parse_datetime($start);
     my $end_obj   = DateTime::Format::ISO8601->parse_datetime($end);
     my $dur_obj   = $end_obj->subtract_datetime($start_obj);
     my $duration  = eval($dur_obj->{'minutes'}*60+$dur_obj->{'seconds'});
-    while (defined $data->[$samples]->{'consumptionPower'}) {
-        $power=$power+$data->[$samples]->{'consumptionPower'};
+    
+    while (defined $data->[$samples]->{'generationPower'}) {
+        $power = $power + $data->[$samples]->{'generationPower'};
         $samples++;
     }
     
@@ -301,6 +350,85 @@ sub get_kwh {
 }
 
 
+#*****************************************************************
+
+=head2 get_energy_consumed - energy consumed for the specified period
+
+ Calculates the total energy consumed over the period specified.
+
+   $NeurioTools->get_energy_consumed($start,$granularity,$end,$frequency);
+ 
+   This method accepts the following parameters:
+     - start       : yyyy-mm-ddThh:mm:ssZ - Required
+                     specified using ISO8601 format
+     - granularity : seconds|minutes|hours|days - Required
+     - end         : yyyy-mm-ddThh:mm:ssZ - Optional
+                     specified using ISO8601 format
+     - frequency   : an integer - Optional
+ 
+ Returns the energy on success 
+ Returns 0 on failure
+=cut
+sub get_energy_consumed {
+    my ($self,$start,$granularity,$end,$frequency) = @_;
+    my $samples = 0;
+    my $energy   = 0;
+    my $kwh;
+    
+    my $data      = $self->{'neurio'}->fetch_Samples($start,$granularity,$end,$frequency);
+    my $start_obj = DateTime::Format::ISO8601->parse_datetime($start);
+    my $end_obj   = DateTime::Format::ISO8601->parse_datetime($end);
+    my $dur_obj   = $end_obj->subtract_datetime($start_obj);
+    my $duration  = eval($dur_obj->{'minutes'}*60+$dur_obj->{'seconds'});
+    
+    while (defined $data->[$samples]->{'consumptionEnergy'}) {
+        $energy = $energy + $data->[$samples]->{'consumptionEnergy'};
+        $samples++;
+    }
+    
+    return $energy;
+}
+
+
+#*****************************************************************
+
+=head2 get_power_consumed - power consumed for the specified period
+
+ Calculates the total power  consumed over the period specified.
+
+   $NeurioTools->get_energy_consumed($start,$granularity,$end,$frequency);
+ 
+   This method accepts the following parameters:
+     - start       : yyyy-mm-ddThh:mm:ssZ - Required
+                     specified using ISO8601 format
+     - granularity : seconds|minutes|hours|days - Required
+     - end         : yyyy-mm-ddThh:mm:ssZ - Optional
+                     specified using ISO8601 format
+     - frequency   : an integer - Optional
+ 
+ Returns the energy on success 
+ Returns 0 on failure
+=cut
+sub get_power_consumed {
+    my ($self,$start,$granularity,$end,$frequency) = @_;
+    my $samples = 0;
+    my $power   = 0;
+    
+    my $data      = $self->{'neurio'}->fetch_Samples($start,$granularity,$end,$frequency);
+    my $start_obj = DateTime::Format::ISO8601->parse_datetime($start);
+    my $end_obj   = DateTime::Format::ISO8601->parse_datetime($end);
+    my $dur_obj   = $end_obj->subtract_datetime($start_obj);
+    my $duration  = eval($dur_obj->{'minutes'}*60+$dur_obj->{'seconds'});
+    
+    while (defined $data->[$samples]->{'consumptionPower'}) {
+        $power = $power + $data->[$samples]->{'consumptionPower'};
+        $samples++;
+    }
+    
+    return $power;
+}
+
+
 #*****************************************************************
 
 =head1 AUTHOR
